
/*
	OneStroke -- onestroke.h
	Copyright 2007 Michael Levin <risujin@risujin.org>

	NOTE: This project has been succeeded by CellWriter, a grid-entry
	      handwriting input panel. Please see:
	        http://risujin.org/cellwriter
*/

/*
    This file is part of OneStroke.

    OneStroke is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OneStroke is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OneStroke; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef OS_VERSION
#warning Define OS_VERSION in the Makefile.
#define OS_VERSION "0.8"
#endif


/*
	Debugging
*/

#ifdef _DEBUG

	/* Debug print */
	void _debug(char* file,int line,char* fmt,...);
	#define debug(format,...)\
		_debug(__FILE__,__LINE__,format,##__VA_ARGS__)

	/* Print errno */
	void _perrno(char* file,int line);
	#define perrno() _perrno(__FILE__,__LINE__)

	/* Assert */
	void _assert(char* file,int line,char* text);
	#define assert(s) if(!(s)) _assert(__FILE__,__LINE__,#s)

#else

	/* Blanks for release builds */
	#define debug(...)
	#define perrno(...)
	#define assert(s)

#endif


/*
	Symbols
*/

/* Scale of point data */
#define OS_SCALE		255

/* Key modifier flags */
#define OS_ALT			1
#define	OS_CTRL			4
#define OS_SHIFT		8
#define OS_WIN			16
#define OS_META			16
#define OS_SUPER		32
#define OS_HYPER		64

/* Point structure */
typedef struct {
	unsigned char x,y;
} osPoint;

/* Segment structure */
typedef struct {
	float length,angle;
} osSegment;

/* Symbol structure */
typedef struct _osSymbol {
	struct _osSymbol *prev,*next;
	osPoint *points;
	osSegment *segments;
	float distance,direction;
	int length,key;
	short flags;
} osSymbol;

/* Match mode */
#define OS_TRAP_DIST	0
#define OS_TRAP_ANGLE	1
#define OS_TRAP_COMP	2
#define	OS_MODES		3
#define	OS_MODE_MASK	3
#define OS_ROTATE		4
#define OS_FAVOR		8
#define OS_TOGGLE_MASK	12
extern int os_mode;

/* Match functions */
float osSymNodeDist(osSymbol *a,osSymbol *b);
float osSymTrapezoid(osSymbol *a,osSymbol *b);
float osSymTrapAngle(osSymbol *a,osSymbol *b);
float osSymTrapComp(osSymbol *a,osSymbol *b);

/* Manipulation functions */
osSymbol* osSymNew(void);
void osSymInit(osSymbol* symbol);
void osSymFree(osSymbol* symbol);
void osSymTrain(osSymbol* symbol);
void osSymKeyChange(osSymbol* symbol,int key,short flags);
char* osSymKeyString(osSymbol* symbol);
void osSymProcess(osSymbol *symbol,float *cx,float *cy);

/* Debug functions */
#ifdef _DEBUG
void osSymDump(osSymbol* sym);
inline static int _osSymValid(char* file,int line,osSymbol* symbol) {
	if(!symbol) {
		_debug(file,line,"Null symbol.");
		return 0;
	}
	return 1;
}
#define osSymValid(s) _osSymValid(__FILE__,__LINE__,(s))
#else
#define osSymValid(s) (s)
#endif
void osSymValidate(osSymbol *symbol);


/*
	Symbol Set
*/

osSymbol* osSymsetFirst(void);
osSymbol* osSymsetLast(void);
void osSymsetFree(void);
void osSymsetAdd(osSymbol* symbol);
#define osSymsetPrev(s) ((s)->prev)
#define osSymsetNext(s) ((s)->next)


/*
	Current Stroke
*/

/* Matched stroke center offset */
float os_match_cx,os_match_cy;

/* Current symbol */
osSymbol *os_cur_stroke;

void osCurDraw(osPoint p);
void osCurReset(void);
osSymbol* osCurMatch(void);


/*
	Profile
*/

/* Frontend flags */
extern int os_flags,os_modified,os_exists;

void osProfileSet(char *path);
int osProfileLoad(void);
int osProfileSave(void);
void osProfileDelete(void);
char* osGetProfile(void);
int osSetCWD(void);


/*
	Frontend
*/

void osFrontend(int argc,char *argv[]);

/* Fatal error */
void _fatal(char* file,int line,char *format,...);
#define fatal(format,...) _fatal(__FILE__,__LINE__,format,##__VA_ARGS__)

